/* global afterAll beforeAll describe expect jest test */
import * as fs from 'node:fs/promises'
import * as fsPath from 'node:path'

import { CredentialsDB } from '@liquid-labs/liq-credentials-db'

import { setupCredentials } from '../setup-credentials'

jest.mock('@liquid-labs/github-toolkit', () => ({
  checkGitHubAPIAccess : () => true,
  checkGitHubSSHAccess : () => true
}))

describe('setupCredentials', () => {
  const testDataDir = fsPath.join(__dirname, 'data')
  const dbPath = fsPath.join(testDataDir, 'test-creds-db.yaml')

  let credentialsDB

  beforeAll(async() => {
    process.env.LIQ_CREDENTIALS_DB_PATH = dbPath

    credentialsDB = new CredentialsDB()
    await setupCredentials({ credentialsDB })
  })

  afterAll(async() => {
    await fs.rm(dbPath)
  })

  test("makes 'GITHUB_SSH' type available", () => expect(credentialsDB.getCredSpec('GITHUB_SSH')).toBeTruthy())

  test("makes 'GITHUB_API' type available", () => expect(credentialsDB.getCredSpec('GITHUB_API')).toBeTruthy())

  describe("'GITHUB_SSH'", () => {
    const credsPath = fsPath.join(testDataDir, 'fake-ssh-key')

    test('can be imported', async() => {
      await credentialsDB.import({ key : 'GITHUB_SSH', noVerify : true, srcPath : credsPath })
      expect(credentialsDB.detail('GITHUB_SSH')).toBeTruthy()
    })

    test('can be verified', () => {
      const fails = credentialsDB.verifyCreds({ keys : ['GITHUB_SSH'] })
      expect(fails).toHaveLength(0)
    })
  })

  describe("'GITHUB_API'", () => {
    const credsPath = fsPath.join(testDataDir, 'fake-github-api-token.yaml')

    test('can be imported', async() => {
      await credentialsDB.import({ key : 'GITHUB_API', noVerify : true, srcPath : credsPath })
      expect(credentialsDB.detail('GITHUB_API')).toBeTruthy()
    })

    test('token can be retrieved', async() => {
      expect(await credentialsDB.getToken('GITHUB_API')).toBe('abcd123')
    })

    test('can be verified', () => {
      const fails = credentialsDB.verifyCreds({ keys : ['GITHUB_API'] })
      expect(fails).toHaveLength(0)
    })
  })
})
